/* 
 *  D-Test
 *
 *  D-Test is a helpful DOS testing program similar to UNIX TEST.
 *  However, attempts have not been made to clone the unix version.
 *  This is the PC version and was designed to augment the miserable
 *  batch file langage MS continues produce.
 *  
 *  This is FREEWARE.  Use it as much as you want, I don't care.
 *  I complied it using borland C, but you shouldn't have a problem
 *  with other compilers.
 *  The complier options for the executable were:
 *  bcc -mt -lt -G -d dtest.c
 *  Sorry, I love COM files.  Takes me back to the good ole' days.
 *
 *  Hope you find it useful.
 *
 *  Send any questions, comments, sugestions, etc, to:
 *
 *  Brett McNamara
 *  ursus@cnj.digex.net
 *
 *  Oh, yeah.  As usual, you use this at your own risk.  I take no
 *  resposibilty for what the computer gremlins may do while you are
 *  running this program.
 *
 */

#include <dos.h>
#include <dir.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/*
 * It's nice to have this at the top.  It sucks to have to change it
 * throughout the program later.
 */
#define progtitle "D-Test v1.0 1994(cw) by Brett McNamara. Freeware.\n"
#define progname "dtest"

#define true  1
#define false 0

/*
 *  DOS File attributes
 *
 *  These are found all over the place, in several header files, under
 *  various names.  Still, it helps to see what you're getting.
 */
#define a_normal   0x00
#define a_readonly 0x01
#define a_hidden   0x02
#define a_system   0x04
#define a_volume   0x08
#define a_dir      0x10
#define a_arch     0x20
#define a_all      0x3f


/*
 *  These are all the switches.  
 *  
 *  Note that the compound switches also have simpler ones.  This
 *  make the computer logic simpiler and quicker.
 */

#define e_val   0
#define s_val   1
#define z_val   2
#define ad_val  3
#define ar_val  4
#define ah_val  5
#define as_val  6
#define aa_val  7
#define d_val   8
#define h_val   9
#define v_val  10
#define a_val  11

#define max_v  12  /* max values */
#define val_u   9  /* end of unique values */

/*
 *  Prototyping, this gives some compliers that warm fuzzy feeling.
 */
void error_halt_c(char *error_str, char error_c);
void error_halt(char *error_str, char *error_file);
void bool_halt(int bool_val);
void usage(void);
void clean_path(char *inpath, char *outpath);
int ismask(char *s);
void get_fname(char *path, char *outpath);
void get_path(char *path, char *outpath);
void readcomline(int argc, char *argv[]);

/*
 *   Global Variable declarations
 */
int flags[max_v] = {false, false, false, false, false, 
		    false, false, false, false, false, 
		    false, false};

char file_name[MAXPATH];


/*
 *   Routines
 */

void usage(void)
   {
   char rdun1[]="file exists";

   printf(progtitle);
   printf("Usage:                              Switches:\n");
   printf("  dtest [switches] filename           -e   %s\n", rdun1);
   printf("                                      -z   %s & 0 length\n", rdun1);
   printf("Examples:                             -s   %s & not 0 length\n", rdun1);
   printf("   dtest -d d:                        -ad  %s & directory\n", rdun1);
   printf("   (if errorlevel 1 goto no_drive)    -ar  %s & read-only\n", rdun1);
   printf("   dtest -ev c:\\autoexec.bat          -ah  %s & hidden\n", rdun1);
   printf("                                      -as  %s & system\n", rdun1);
   printf("Returns:                              -aa  %s & archive\n", rdun1);
   printf("   0 if condition is true             -d   drive exists\n");
   printf("   1 if condition is false            -v   verbose\n");
   printf("   2 if invalid syntax                -h,? help\n");
   }

void error_halt_c(char *error_str, char error_c)
   {
   if (flags[v_val]) {
      printf("%s err: ", progname);
      printf(error_str, error_c);
      printf("\n");
      printf("for help, type: dtest -h\n");
      }
   exit(2);
   }

void error_halt(char *error_str, char *error_file)
   {
   if (flags[v_val]) {
      printf("%s err: ", progname);
      printf(error_str, error_file);
      printf("\n");
      printf("for help, type: dtest -h\n");
      }
   exit(2);
   }

void bool_halt(int bool_val)
   {
   if (bool_val) {
      if (flags[v_val]) printf("dtest: true\n");
      exit(0);
      }
   if (flags[v_val]) printf("dtest: false\n");
   exit(1);
   }

/*
 *  This is a single condition testing program. 
 *  Wild cards can ruin your day.
 */
int ismask(char *s)
   {
   int i = 0;
   while ( (s[i] != '*') && (s[i] != '?') && (s[i] != 0) ) i++;
   return(s[i] != 0);
   }


/*
 *  Throw all the swiches, we'll work it out later.
 */
void readcomline(int argc, char *argv[])
   {
   char tmp[MAXPATH];
   int cparm;
   int i,j;
   int switch_found = false;
   int files_found = 0;

   for ( cparm=1; cparm<argc; cparm++) {
      strcpy(tmp, argv[cparm]); strlwr(tmp);
      if ( ( tmp[0] == '-' ) || ( tmp[0] == '/' ) ) {
	 switch_found = true;
	 for (j=1 ,i=strlen(tmp)-1; j<=i; j++) {
	    switch(tmp[j]) {
	       case 'e': flags[e_val] = true; break;
	       case 's': flags[s_val] = true; break;
	       case 'z': flags[z_val] = true; break;
	       case 'v': flags[v_val] = true; break;
	       case 'h': flags[h_val] = true; break;
	       case '?': flags[h_val] = true; break;
	       case 'd': flags[d_val] = true; break;

	       case 'a': 
		  flags[a_val] = true;
		  if (j != i) {
		     j++;
		     switch(tmp[j]) {
			case 'd': flags[ad_val] = true; break;
			case 'r': flags[ar_val] = true; break;
			case 'h': flags[ah_val] = true; break;
			case 's': flags[as_val] = true; break;
			case 'a': flags[aa_val] = true; break;
			default:
			   error_halt_c("invalid flag 'a%c'",tmp[j]);
			}
		     }
		  else
		     error_halt("invalid flag 'a '","");
		  break;
	       
	       default: error_halt_c("invalid flag '%c'", tmp[j]);
	       }
	    }

	 }
      else 
	 if (files_found == 0) {
	    strcpy(file_name, tmp);
	    files_found++;
	    }
	 else error_halt("too many filenames", "");
      }
   if ( flags[h_val] ) { usage(); exit(0); };
   if ( !switch_found ) error_halt("nothing to process", "");
   if ( files_found == 0 ) error_halt("nothing to process", "");
   if ( ismask(file_name) ) error_halt("wild cards not accepted", "");
   switch_found = 0;
   for(i=0; i<val_u; i++)
      if (flags[i])
	 if (++switch_found > 1 )
	    error_halt("too many switches", "");
   }


int main(int argc, char *argv[])
   {
   struct ffblk dta;
   struct diskfree_t df;
   int temp_bool;
   
   if ( argc < 2 ) error_halt("nothing to process", "");
   
   readcomline(argc, argv);
   
   if (flags[d_val]) {
      temp_bool = (_dos_getdiskfree(file_name[0]-'a'+1, &df) == 0);
      bool_halt(temp_bool);
      }
   
   temp_bool = (findfirst(file_name, &dta, a_all ) == 0);
   if (flags[e_val]) bool_halt(temp_bool);
   
   if (flags[a_val]) {
      if (flags[ad_val]) bool_halt( (dta.ff_attrib & a_dir) );
      if (flags[ar_val]) bool_halt( (dta.ff_attrib & a_readonly) );
      if (flags[ah_val]) bool_halt( (dta.ff_attrib & a_hidden) );
      if (flags[as_val]) bool_halt( (dta.ff_attrib & a_system) );
      if (flags[aa_val]) bool_halt( (dta.ff_attrib & a_arch) );
      }

   temp_bool = ( dta.ff_fsize == 0);
   if (flags[z_val]) bool_halt(temp_bool);
   if (flags[s_val]) bool_halt(!temp_bool);

   return(0);
   }
